@@ -1,5 +1,5 @@
 # testing/requirements.py
-# Copyright (C) 2005-2017 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2018 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -15,6 +15,8 @@ to provide specific inclusion/exclusions.
 
 """
 
+import sys
+
 from . import exclusions
 from .. import util
 
@@ -110,6 +112,32 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def parens_in_union_contained_select_w_limit_offset(self):
+        """Target database must support parenthesized SELECT in UNION
+        when LIMIT/OFFSET is specifically present.
+
+        E.g. (SELECT ...) UNION (SELECT ..)
+
+        This is known to fail on SQLite.
+
+        """
+        return exclusions.open()
+
+    @property
+    def parens_in_union_contained_select_wo_limit_offset(self):
+        """Target database must support parenthesized SELECT in UNION
+        when OFFSET/LIMIT is specifically not present.
+
+        E.g. (SELECT ... LIMIT ..) UNION (SELECT .. OFFSET ..)
+
+        This is known to fail on SQLite.  It also fails on Oracle
+        because without LIMIT/OFFSET, there is currently no step that
+        creates an additional subquery.
+
+        """
+        return exclusions.open()
+
     @property
     def boolean_col_expressions(self):
         """Target database must support boolean expressions as columns"""
@@ -259,6 +287,14 @@ class SuiteRequirements(Requirements):
 
         return exclusions.closed()
 
+    @property
+    def server_side_cursors(self):
+        """Target dialect must support server side cursors."""
+
+        return exclusions.only_if([
+            lambda config: config.db.dialect.supports_server_side_cursors
+        ], "no server side cursors support")
+
     @property
     def sequences(self):
         """Target database must support SEQUENCEs."""
@@ -313,6 +349,10 @@ class SuiteRequirements(Requirements):
     def foreign_key_constraint_reflection(self):
         return exclusions.open()
 
+    @property
+    def foreign_key_constraint_option_reflection(self):
+        return exclusions.closed()
+
     @property
     def temp_table_reflection(self):
         return exclusions.open()
@@ -341,6 +381,11 @@ class SuiteRequirements(Requirements):
         """target dialect supports reflection of unique constraints"""
         return exclusions.open()
 
+    @property
+    def check_constraint_reflection(self):
+        """target dialect supports reflection of check constraints"""
+        return exclusions.closed()
+
     @property
     def duplicate_key_raises_integrity_error(self):
         """target dialect raises IntegrityError when reporting an INSERT
@@ -460,6 +505,19 @@ class SuiteRequirements(Requirements):
 
         return exclusions.open()
 
+    @property
+    def json_type(self):
+        """target platform implements a native JSON type."""
+
+        return exclusions.closed()
+
+    @property
+    def json_array_indexes(self):
+        """"target platform supports numeric array indexes
+        within a JSON structure"""
+
+        return self.json_type
+
     @property
     def precision_numerics_general(self):
         """target backend has general support for moderately high-precision
@@ -607,7 +665,7 @@ class SuiteRequirements(Requirements):
 
             select data as foo from test order by foo || 'bar'
 
-        Lots of databases including Postgresql don't support this,
+        Lots of databases including PostgreSQL don't support this,
         so this is off by default.
 
         """
@@ -664,12 +722,50 @@ class SuiteRequirements(Requirements):
             "Stability issues with coverage + py3k"
         )
 
+    @property
+    def python2(self):
+        return exclusions.skip_if(
+            lambda: sys.version_info >= (3,),
+            "Python version 2.xx is required."
+        )
+
+    @property
+    def python3(self):
+        return exclusions.skip_if(
+            lambda: sys.version_info < (3,),
+            "Python version 3.xx is required."
+        )
+
+    @property
+    def cpython(self):
+        return exclusions.only_if(
+            lambda: util.cpython,
+            "cPython interpreter needed"
+        )
+
+    @property
+    def non_broken_pickle(self):
+        from sqlalchemy.util import pickle
+        return exclusions.only_if(
+            lambda: not util.pypy and pickle.__name__ == 'cPickle'
+                or sys.version_info >= (3, 2),
+            "Needs cPickle+cPython or newer Python 3 pickle"
+        )
+
+    @property
+    def predictable_gc(self):
+        """target platform must remove all cycles unconditionally when
+        gc.collect() is called, as well as clean out unreferenced subclasses.
+
+        """
+        return self.cpython
+
     @property
     def no_coverage(self):
         """Test should be skipped if coverage is enabled.
 
         This is to block tests that exercise libraries that seem to be
-        sensitive to coverage, such as Postgresql notice logging.
+        sensitive to coverage, such as PostgreSQL notice logging.
 
         """
         return exclusions.skip_if(
